﻿// LocalSyncBehavior.cs
//

using System;
using System.Windows;
using System.Windows.Ria.Data;
using System.Windows.Ria.Sync.Offline;

namespace System.Windows.Ria.Sync {

    public sealed class LocalSyncBehavior<TDomainContext> : DomainSyncBehavior<TDomainContext> where TDomainContext : DomainContext {

        private string _isoStorageFileName;

        public LocalSyncBehavior(TimeSpan syncInterval, string isoStorageFileName)
            : base(syncInterval) {
            if (String.IsNullOrEmpty(isoStorageFileName)) {
                throw new ArgumentNullException("isoStorageFileName");
            }

            _isoStorageFileName = isoStorageFileName;
        }

        protected override void OnAttach() {
            StartSync();
        }

        protected override void OnDetach() {
            StopSync();
        }

        protected internal override void Sync(bool performLoad, bool performSave) {
            if (performSave) {
                MetaEntityStorage.Save(DomainContext, _isoStorageFileName);
                StartSync();
            }
            else if (performLoad) {
                MetaEntityStorage.Load(DomainContext, _isoStorageFileName);
                StartSync();
            }
        }
    }
}
