﻿// MetaEntitySerializer.cs
//

using System;
using System.Collections.Generic;
using System.IO;
using System.Runtime.Serialization.Json;
using System.Text;

namespace System.Windows.Ria.Sync.Offline {

    internal class MetaEntitySerializer {

        private static IEnumerable<Type> GetKnownTypes(IEnumerable<Type> entityTypes) {
            List<Type> knownTypes =
                new List<Type>() {
                    typeof(MetaEntity),
                    typeof(MetaEntityAction),
                    typeof(List<object>)
                };
            knownTypes.AddRange(entityTypes);

            return knownTypes;
        }

        internal static string Serialize(List<List<MetaEntity>> list, IEnumerable<Type> entityTypes) {
            DataContractJsonSerializer serializer =
                new DataContractJsonSerializer(list.GetType(), GetKnownTypes(entityTypes));

            MemoryStream ms = new MemoryStream();
            serializer.WriteObject(ms, list);
            ms.Seek(0, SeekOrigin.Begin);

            string json = null;
            using (StreamReader reader = new StreamReader(ms)) {
                json = reader.ReadToEnd();
            }

            return json;
        }

        internal static List<List<MetaEntity>> Deserialize(string jsonData, IEnumerable<Type> entityTypes) {
            DataContractJsonSerializer serializer =
                new DataContractJsonSerializer(typeof(List<List<MetaEntity>>), GetKnownTypes(entityTypes));

            MemoryStream ms = new MemoryStream(Encoding.Unicode.GetBytes(jsonData));
            return ((List<List<MetaEntity>>)serializer.ReadObject(ms));
        }
    }
}